<?php

namespace FiloBlu\Cegid\Helper\Cegid;

// Filoblu
use FiloBlu\Cegid\Helper\Cegid\Base as BaseHelper;

// Magento
use Magento\Customer\Api\CustomerRepositoryInterface;
use Magento\Sales\Api\Data\OrderInterface;

// Services
use FiloBlu\CegidY2\Service\CredentialProvider;
use FiloBlu\CegidY2\Service\ServiceProvider;
//use FiloBlu\CegidY2\Api\SaleDocumentService\ServiceType\Close as ServiceClose;
//use FiloBlu\CegidY2\Api\SaleDocumentService\ServiceType\CloseFactory as ServiceProvider;

// All struct types
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\RetailContext;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\RetailContextFactory;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\Close;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\CloseFactory;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\Close_RequestFactory as CloseRequestFactory;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\SaleDocumentIdentifier;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\SaleDocumentIdentifierFactory;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\SaleDocumentReference;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\SaleDocumentReferenceFactory;
use FiloBlu\CegidY2\Api\SaleDocumentService\EnumType\SaleDocumentType;

use FiloBlu\Cegid\Helper\Cegid\Order as OrderHelper;

use RuntimeException;

/**
 * Class OrderClose
 * @package FiloBlu\Cegid\Helper\OrderClose
 */
class OrderClose extends BaseHelper {

    /**
     * @var CredentialProvider
     */
    protected $credentialProvider;
    /**
     * @var ServiceProvider
     */
    protected $serviceProvider;
    /**
     * @var ServiceProvider
     */
    protected $closeService;
    /**
     * @var SaleDocumentReference
     */
    protected $saleDocumentReference;
    /**
     * @var SaleDocumentReferenceFactory
     */
    protected $sleDocumentReferenceFactory;
    /**
     * @var CustomerRepositoryInterface
     */
    protected $customerRepositoryInterface;
    /**
     * @var SaleDocumentIdentifier
     */
    protected $saleDocumentIdentifier;
    /**
     * @var SaleDocumentIdentifierFactory
     */
    protected $saleDocumentIdentifierFactory;
    /**
     * @var CloseRequestFactory
     */
    protected $closeRequestFactory;
    /**
     * @var Close
     */
    protected $close;
    /**
     * @var CloseFactory
     */
    protected $closeFactory;
    /**
     * @var RetailContext
     */
    protected $retailContext;
    /**
     * @var RetailContextFactory
     */
    protected $retailContextFactory;
    /**
     * @var Order
     */
    protected $orderHelper;

    protected $order_source_items_quantities;
    protected $current_order;
    protected $current_source_code;
    protected $cegid_customer_id;

    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        CredentialProvider $credentialProvider,
        CloseFactory $closeFactory,
        Close $close,
        ServiceProvider $closeService,
        SaleDocumentReference $saleDocumentReference,
        SaleDocumentReferenceFactory $saleDocumentReferenceFactory,
        CustomerRepositoryInterface $customerRepositoryInterface,
        SaleDocumentIdentifier $saleDocumentIdentifier,
        SaleDocumentIdentifierFactory $saleDocumentIdentifierFactory,
        CloseRequestFactory $closeRequestFactory,
        RetailContext $retailContext,
        RetailContextFactory $retailContextFactory,
        OrderHelper $orderHelper
    )
    {
        parent::__construct($context);
        //$this->serviceProvider = $serviceProvider;
        $this->closeFactory = $closeFactory;
        $this->close = $close;
        $this->saleDocumentReference = $saleDocumentReference;
        $this->saleDocumentReferenceFactory = $saleDocumentReferenceFactory;
        $this->customerRepositoryInterface = $customerRepositoryInterface;
        $this->saleDocumentIdentifier = $saleDocumentIdentifier;
        $this->saleDocumentIdentifierFactory = $saleDocumentIdentifierFactory;
        $this->closeRequestFactory = $closeRequestFactory;
        $this->retailContext = $retailContext;
        $this->retailContextFactory = $retailContextFactory;
        $this->credentialProvider = $credentialProvider;
        $this->closeService = $closeService;
        $this->orderHelper = $orderHelper;
    }

    public function initOrderClose(OrderInterface $order){
        $this->order_source_items_quantities = $this->orderHelper->getOrderSourceItemsQuantities($order);
        $this->cegid_customer_id = $this->orderHelper->getCustomerCegidFromOrder($order);
        $this->current_order = $order;
        $this->current_source_code;
    }

    public function createCloses(OrderInterface $order){

        $this->initOrderClose($order);

        $errors = array();
        foreach ($this->order_source_items_quantities as $source_code => $fb_order_item) {
            $this->current_source_code = $source_code;
            $success = $this->createClose($order);

            if ($success !== true){
                $errors[] = $success;
            }
        }

        if (!empty($errors)){
            throw new RuntimeException(implode("\n", $errors));
        }
        return true;

    }

    public function createClose(OrderInterface $order){

        $parameters = $this->closeFactory->create();
        $parameters->setCloseRequest($this->getCloseRequest($order));
        $parameters->setClientContext($this->getRetailContext());

        // Execute
        $closeService = $this->closeService->create();
        $closeService->Close($parameters);
        // Save data
        $request = $closeService->getLastRequest();
        $response = $closeService->getLastResponse();
        $error = array_values($closeService->getLastError());
        $result = $closeService->getResult();

        $this->writeLog('createClose',$order,$request,$response);

        if(!$result && $error && count($error)){
            return $error[0]->getMessage();
        }
        return true;

        //throw new RuntimeException(implode("\n", $request->getLastError()));
    }

    /**
     * @param OrderInterface $order
     * @return mixed
     */
    protected function getCloseRequest(OrderInterface $order){
        $closeRequest = $this->closeRequestFactory->create();
        $closeRequest->setIdentifier($this->getIdentifier($order));
        return $closeRequest;
    }

    /**
     * @param OrderInterface $order
     * @return mixed
     */
    protected function getIdentifier(OrderInterface $order){
        $identifier = $this->saleDocumentIdentifierFactory->create();
        $identifier->setReference($this->getReference($order));
        return $identifier;
    }

    /**
     * @param OrderInterface $order
     * @return mixed
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    protected function getReference(OrderInterface $order){

        $customer = $this->customerRepositoryInterface->getById($order->getCustomerId());

        $saleDocumentReference = $this->saleDocumentReferenceFactory->create();
        $saleDocumentReference->setCustomerId($this->cegid_customer_id);
        $saleDocumentReference->setInternalReference($this->orderHelper->getOrderReference($order,$this->current_source_code));
        $saleDocumentReference->setType(SaleDocumentType::VALUE_CUSTOMER_ORDER);

        return $saleDocumentReference;
    }

    /**
     * @return mixed
     */
    public function getRetailContext()
    {
        $retailContext = $this->retailContextFactory->create([
            'databaseId' => $this->credentialProvider->getDatabase('sale_document_service')
        ]);
        return $retailContext;
    }

}